/*
 *	Qizx/Open version 0.4
 *
 *	Copyright (c) 2003-2004 Xavier C. FRANC -- All rights reserved.
 *
 *	This program is free software; you can redistribute it  and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation (see LICENSE.txt).
 */


package net.xfra.qizxopen.xquery.fn;
import net.xfra.qizxopen.xquery.impl.*;

import net.xfra.qizxopen.util.time.*;
import net.xfra.qizxopen.xquery.*;
import net.xfra.qizxopen.xquery.dt.MomentValue;
import net.xfra.qizxopen.xquery.dt.DurationValue;

/**
 *  Implementation of function fn:get-year-from-dateTime.
 */
public class GetYearFromDateTime extends Function {

    static Prototype[] protos = { 
      Prototype.fn("get-year-from-dateTime", Type.INTEGER.opt, Exec.class).arg("op", Type.DATE_TIME.opt)
    };
    public Prototype[] getProtos() { return protos; }

    public static class Exec extends Function.IntegerCall {
        public long evalAsInteger(Focus focus, EvalContext context) throws XQueryException {
            context.at(this);
	    Item v = args[0].evalAsOptItem(focus, context);
	    if(v == null)
		throw EmptyException.allowed();
	    if(v instanceof MomentValue) {
		MomentValue mv = (MomentValue) v;
		return mv.getValue().getYear();
	    }
	    try {
		DateTime dt = DateTime.parseDateTime( v.asString(), context.getImplicitTimezone() );
		return dt.getYear();
	    }
	    catch(DateTimeException e) {
		context.error(this, "cannot cast to xs:dateTime : "+ e.getMessage()); return 0;
	    }
        }
    }
}
